import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.pj_base
import SEP.pf_copy
import SEP.pf_irreg
import SEP.pf_split
import SEP.paths
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.09.29"


class inv(SEP.pj_base.par_job):
  def __init__(self,name="AMO_to_zero"):
    """Initialize the AMO  program"""
    SEP.pj_base.par_job.__init__(self,name)
    self.add_doc_params()
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.pj_base.par_job.add_doc_params(self)

    self.add_doc_param("stdin",doc="Input file")
    self.add_doc_param("stdout",doc="Output file")

    self.add_doc_param("maxsize",1000,doc="Maximum amount of memory to use")

    self.add_doc_param("nblock",doc="Number of blocks",required=None)
    self.add_doc_param("f_max",60.,doc="Maximum frequency")
    self.add_doc_param("t_min_log",.2,doc="Minimum time for logstrech")
    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.add_doc_param("rho",.85,"Epsilon for normalization calculation")
    self.add_doc_param("eps0",.000001,"Epsilon for normalization calculation")
    self.add_doc_param("deps",.000001,"Epsilon for normalization calculation")
    self.add_doc_param("niter",20,doc="Number of iteratons")
    self.add_doc_param("amx_n",doc="Number of samples along midpoint x")
    self.add_doc_param("amy_n",doc="Number of samples along midpoint y")
    self.add_doc_param("ahx_n",doc="Number of samples along half-offset x")
    self.add_doc_param("ahy_n",doc="Number of samples along half-offset y")
    self.add_doc_param("small_amp_ifreq",9999,doc="Number of samples in time")
    self.add_doc_param("at__n",doc="Number of samples in time")
                                                                                             
    self.add_doc_param("amx_o",doc="First sample along midpoint x")
    self.add_doc_param("amy_o",doc="First sample along midpoint y")
    self.add_doc_param("ahx_o",doc="First sample along half-offset x")
    self.add_doc_param("ahy_o",doc="First sample along half-offset y")
    self.add_doc_param("at__o",doc="First sample in time")
                                                                                             
    self.add_doc_param("amx_d",doc="Sampling along midpoint x")
    self.add_doc_param("amy_d",doc="Sampling along midpoint y")
    self.add_doc_param("ahx_d",doc="Sampling along half-offset x")
    self.add_doc_param("ahy_d",doc="Sampling along half-offset y")
    self.add_doc_param("at__d",doc="Sampling along time axis")
    self.add_doc_param("ratio_end",.5,doc="Maximum ratio decrease in residual")


    self.param_list=["niter","verb","ratio_end","rho","small_amp_ifreq"]
    self.param_list.extend(["t_min_log","f_max","eps0","deps"])
    self.param_list.extend(["amx_n","amx_o","amx_d"])
    self.param_list.extend(["amy_n","amy_o","amy_d"])
    self.param_list.extend(["ahx_n","ahx_o","ahx_d"])
    self.param_list.extend(["ahy_n","ahy_o","ahy_d"])
    self.param_list.extend(["at__n","at__o","at__d"])

    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")

    ins=SEP.sepfile.sep_file(self.param("stdin"))

    nblock=int(self.param("nblock",default=ins.axis(2)[0]))
   


    #get the section parameters
    sect_pars=self.build_sect_params(ins,nblock)
    self.add_param("program",self.prog)

    self.add_param("pverb",self.param("verb") )
    self.par_files={}


    self.par_files["intag"]=SEP.pf_split.parfile(name=self.param("stdin"), 
      dff_axis=2, tag="<",usage="INPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    self.par_files["headers"]=SEP.pf_copy.parfile(name=self.param("headers"),
      tag="headers=",usage="INPUT",njobs=nblock,
      nblock=nblock)

    self.par_files["outtag"]=SEP.pf_split.parfile(name=self.param("stdout"), 
      dff_axis=5, tag=">",usage="OUTPUT",njobs=nblock,
      restart=restart,nblock=nblock)

    #self.par_files["res"]=SEP.pf_split.parfile(name="res.H", 
    #  dff_axis=5, tag="res",usage="OUTPUT",njobs=nblock,
    #  restart=restart,nblock=nblock)

    self.add_param("files",self.par_files)
    order=[]
    for i in range(nblock): order.append(str(i))
    self.add_param("order",order)
    self.add_param("sect_pars",sect_pars)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.pj_base.par_job.prep_run(self,restart)
    

  def build_sect_params(self,ins,nblock):
    """Build parameters for parallel job"""
    sect_pars={}

    o=ins.axis(5)[1]
    d=ins.axis(5)[2]
    ntot=ins.axis(5)[0]
    nsmall=int(ins.axis(5)[0]/nblock)
    nremain=ntot-nsmall*nblock;
    for i in range(nblock):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
      if i < nremain: n=nsmall+1
      else: n=nsmall
      o+=d
    return sect_pars
