import SEP.sepfile
import SEP.util
import SEP.spawn
import SEP.paths
import SEP.flow
import SEP.cgstep
import solver
import amo_inv_flow
import SEP.pj_split
import SEP.pj_func
import SEP.pv_always_split
import SEP.pf_split
import SEP.mach_list
import SEP.mach

class no_style_h0(SEP.solv_smp.solver):
  def __init__(self,main_args,interp,prec,zero,name="AMO.INVERSE"):
    self.main=main_args
    self.interp=interp
    self.prec=prec
    self.zero=zero
    SEP.solv_smp.solver.__init__(self,name)

  def add_doc_params(self):
    self.add_param("niter",5,"Number of iterations")


  def return_data(self):
    fname=self.main.param("irregular")
    data=SEP.pv_always_split.cmplx_vector(name="%s.stretch"%fname,
     usage="INPUT",remove=None,tag="data=",load=1,exists=1)
    return data

  def return_full(self,data,restart=None):
    fname=self.main.param("irregular")
    if not  restart:
      full=data.clone_space(name="%s.freqfull"%fname)
      nfreq=data.axis(2)[0]
      full.set_axis(1,self.main.param("amx_n"),self.main.param("amx_o"),self.main.param("amx_d"))
      full.set_axis(2,self.main.param("amy_n"),self.main.param("amy_o"),self.main.param("amy_d"))
      full.set_axis(3,self.main.param("ahx_n"),self.main.param("ahx_o"),self.main.param("ahx_d"))
      full.set_axis(4,self.main.param("ahy_n"),self.main.param("ahy_o"),self.main.param("ahy_d"))
      full.set_axis(5,nfreq,0.,1.)
      full.set_dff_axis(5)
      full.zero()
    else: full=SEP.pv_always_split.cmplx_vector(name="%s.freqfull"%fname,
     usage="INPUT",remove=None,tag="model=",load=1,exists=1)
    return full


  def return_model(self,data,restart=None):
    fname=self.main.param("irregular")
    if not  restart:
      model=data.clone_space(name="%s.freqbig"%fname)
      nfreq=data.axis(2)[0]
      model.set_axis(1,self.main.param("amx_n"),self.main.param("amx_o"),self.main.param("amx_d"))
      model.set_axis(2,self.main.param("amy_n"),self.main.param("amy_o"),self.main.param("amy_d"))
      model.set_axis(3,self.main.param("ahx_n"),self.main.param("ahx_o"),self.main.param("ahx_d"))
      model.set_axis(4,1,0.,self.main.param("ahy_d"))
      model.set_axis(5,nfreq,0.,1.)
      model.set_dff_axis(5)
      model.zero()
    else: model=SEP.pv_always_split.cmplx_vector(name="%s.freqbig"%fname,
     usage="INPUT",remove=None,tag="model=",load=1,exists=1)
    return model

  def prep_run(self,restart=None):
    """ AAAA"""
    self.data=self.return_data()
    self.full=self.return_full(self.data,restart)
    self.model=self.return_model(self.data,restart)

    SEP.mach.mach=SEP.mach_list.machine(self.data.get_stat_elem(1).values())
    self.prec.prep_run(self.model,self.model,restart)
    self.zero.prep_run(self.model,self.full,restart)
    self.interp.prep_run(self.full,self.data,restart)
    self.my_op=SEP.op_combo.chain("op_chain",[self.zero,self.interp],
      verb=self.main.param("verb"))
    self.mycombo=SEP.op_combo.chain("combo_chain",[self.prec,self.my_op],
      verb=self.main.param("verb"))
    self.add_param("op",self.mycombo)

    self.add_param("stepper",SEP.cgstep.cgstep("matcg"))
    self.add_param("niter",self.main.param("niter"))
     
    if not restart:
      self.pvar=self.model.clone_space(name="%s.prec"%(self.model.vec_name()))
      self.pvar.zero() 
    else:
      self.pvar=SEP.pv_always_split.cmplx_vector(name="%s.prec"%self.model.vec_name(),
       usage="INPUT",remove=None,tag="model=",load=1,exists=1)
      
    self.add_param("model",self.pvar)
    self.add_param("data",self.data)
    SEP.solv_smp.solver.prep_run(self,restart)
    self.prec.forward(self.pvar,self.model)





class no_style(SEP.solv_smp.solver):
  def __init__(self,main_args,interp,prec,name="AMO.INVERSE"):
    self.main=main_args
    self.interp=interp
    self.prec=prec
    SEP.solv_smp.solver.__init__(self,name)

  def add_doc_params(self):
    self.add_param("niter",5,"Number of iterations")


  def return_data(self):
    fname=self.main.param("irregular")
    data=SEP.pv_always_split.cmplx_vector(name="%s.stretch"%fname,
     usage="INPUT",remove=None,tag="data=",load=1,exists=1)
    return data

  def return_model(self,data,restart=None):
    fname=self.main.param("irregular")
    if not  restart:
      model=data.clone_space(name="%s.freqbig"%fname)
      nfreq=data.axis(2)[0]
      model.set_axis(1,self.main.param("amx_n"),self.main.param("amx_o"),self.main.param("amx_d"))
      model.set_axis(2,self.main.param("amy_n"),self.main.param("amy_o"),self.main.param("amy_d"))
      model.set_axis(3,self.main.param("ahx_n"),self.main.param("ahx_o"),self.main.param("ahx_d"))
      model.set_axis(4,self.main.param("ahy_n"),self.main.param("ahy_o"),self.main.param("ahy_d"))
      model.set_axis(5,nfreq,0.,1.)
      model.set_dff_axis(5)
      model.zero()
    else: model=SEP.pv_always_split.cmplx_vector(name="%s.freqbig"%fname,
     usage="INPUT",remove=None,tag="model=",load=1,exists=1)
    return model

  def prep_run(self,restart=None):
    """ AAAA"""
    self.data=self.return_data()
    self.model=self.return_model(self.data,restart)

    SEP.mach.mach=SEP.mach_list.machine(self.data.get_stat_elem(1).values())
    self.prec.prep_run(self.model,self.data,restart)
    self.interp.prep_run(self.model,self.data,restart)
    self.mycombo=SEP.op_combo.chain("op_chain",[self.prec,self.interp],
      verb=self.main.param("verb"))
    self.add_param("op",self.mycombo)

    self.add_param("stepper",SEP.cgstep.cgstep("matcg"))
    self.add_param("niter",self.main.param("niter"))
     

    if not restart:
      self.pvar=self.model.clone_space(name="%s.prec"%(self.model.vec_name()))
      self.pvar.zero() 
    else:
      self.pvar=SEP.pv_always_split.cmplx_vector(name="%s.prec"%self.model.vec_name(),
       usage="INPUT",remove=None,tag="model=",load=1,exists=1)
      
    self.add_param("model",self.pvar)
    self.add_param("data",self.data)
#    self.prec.dot_test()
#    self.interp.dot_test() 
#    self.mycombo.dot_test()
#    SEP.util.err("")
    SEP.solv_smp.solver.prep_run(self,restart)
    self.prec.forward(self.pvar,self.model)

class no_style_aoff(no_style):
  def __init__(self,main_args,interp,prec,name="AMO.INVERSE"):
    no_style.__init__(self,main_args,interp,prec,name)

  def return_model(self,data,restart=None):
    fname=self.main.param("irregular")
    if not  restart:
      model=data.clone_space(name="%s.freqbig"%fname)
      nfreq=data.axis(2)[0]
      model.set_axis(1,self.main.param("amx_n"),self.main.param("amx_o"),self.main.param("amx_d"))
      model.set_axis(2,self.main.param("amy_n"),self.main.param("amy_o"),self.main.param("amy_d"))
      model.set_axis(3,self.main.param("ah_n"),self.main.param("ah_o"),self.main.param("ah_d"))
      model.set_axis(4,1,0.,0.)
      model.set_axis(5,nfreq,0.,1.)
      model.set_dff_axis(5)
      model.zero()
    else: model=SEP.pv_always_split.cmplx_vector(name="%s.freqbig"%fname,
     usage="INPUT",remove=None,tag="model=",load=1,exists=1)
    return model
