import SEP.sepfile
import SEP.util
import SEP.spawn
import types,math
import string
import SEP.paths
import SEP.pf_copy
import SEP.pv_always_split
import SEP.op_oc_par_split
import SEP.pj_split
__author__ = "Robert G. Clapp"
__email__ = "bob@sep.stanford.edu"
__version = "2005.08.24"


class  op(SEP.op_oc_par_split.operator):
  def __init__(self,name="AMO_shape"):
    """Initialize the AMO  program"""
    SEP.op_oc_par_split.operator.__init__(self,name,msg="Smoothing")
 
  def add_doc_params(self):
    """Parameters for AMO_to_freq"""
    SEP.op_oc_par_split.operator.add_doc_params(self)


    self.add_doc_param("verb","y",doc="Whether or not to be verbose")
    self.add_doc_param("a1_n",doc="Number of samples along midpoint x")
    self.add_doc_param("a2_n",doc="Number of samples along midpoint y")
    self.add_doc_param("a3_n",doc="Number of samples along half-offset x")
    self.add_doc_param("a4_n",doc="Number of samples along half-offset y")
    self.add_doc_param("at__n",doc="Number of samples in time")
                                                                                
    self.add_doc_param("a1_o",doc="First sample along midpoint x")
    self.add_doc_param("a2_o",doc="First sample along midpoint y")
    self.add_doc_param("a3_o",doc="First sample along half-offset x")
    self.add_doc_param("a4_o",doc="First sample along half-offset y")
    self.add_doc_param("at__o",doc="First sample in time")
                                                                                
    self.add_doc_param("a1_d",doc="Sampling along midpoint x")
    self.add_doc_param("a2_d",doc="Sampling along midpoint y")
    self.add_doc_param("a3_d",doc="Sampling along half-offset x")
    self.add_doc_param("a4_d",doc="Sampling along half-offset y")
    self.add_doc_param("at__d",doc="Sampling along time axis")
    self.add_doc_param("spread","y",doc="Whether or not the data is distributed on the nodes")

    self.add_doc_param("ncmpx",2,doc="Rectangle smoothing ncmpx axis")
    self.add_doc_param("ncmpy",2,doc="Rectangle smoothing ncmpy axis")
    self.add_doc_param("rect1",50,doc="Rectangle smoothing hx axis")
    self.add_doc_param("rect2",5,doc="Rectangle smoothing hy axis")
    self.add_doc_param("f_max",60.,doc="Maximum frequency (60.)")
    self.add_doc_param("t_min_log",doc="Minimum time for log transform",required=None)
    self.param_list=["verb", "f_max","t_min_log"]

    self.param_list.extend(["a1_n","a1_o","a1_d"])
    self.param_list.extend(["a2_n","a2_o","a2_d"])
    self.param_list.extend(["a3_n","a3_o","a3_d"])
    self.param_list.extend(["a4_n","a4_o","a4_d"])
    self.param_list.extend(["at__n","at__o","at__d"])
    self.param_list.extend(["rect1","rect2","ncmpx","ncmpy"])
    self.del_doc_param("adj_com")
    self.delete_object_par_params()


  def prep_run(self,restart=None):  
    """Check to make sure the parameters make sense"""

    restart=self.main.param("restart")
    aa=SEP.stat_sep.status("%s.stat"%self.param("mapping_file",error=1),load=1)
    aa.restart(1)
    self.map=aa.get_stat_elem(1)
                                                                                             
    #get the section parameters
    nblock=len(self.map.keys())


    #get the section parameters
    sect_pars=self.build_sect_params(nblock)
    self.add_param("program","AMO_causint.x")
    self.add_param("adj_com","adj=y")

    self.add_param("pverb",self.param("verb") )
    self.par_files={}

   
    if self.logic("spread"): 
      load=1
      remove=None
    else: 
     load=None
     remove=1

    self.par_files["range"]=self.param("range").clone(tag="data=",space_only=1)

#    SEP.pv_always_split.cmplx_vector(name="interp.range", 
#      dff_axis=5, tag="data=",usage="INPUT",njobs=nblock, 
#     sect_map=self.map,nblock=nblock,remove=remove)

    self.par_files["domain"]=self.param("domain").clone(tag="model=",space_only=1)

#  SEP.pv_always_split.cmplx_vector(name="interp.domain" , 
#        dff_axis=5, tag="model=",usage="OUTPUT",njobs=nblock,
#        sect_map=self.map,nblock=nblock,collect_it=None,remove=remove)

    self.add_param("files",self.par_files)
    order=[]
    for i in range(nblock): order.append(str(i))
    self.add_param("order",order)
    self.add_param("sect_pars",sect_pars)
    self.add_param("global_pars",self.return_set_params(self.param_list))
    SEP.op_oc_par_split.operator.prep_run(self,restart)
    

  def build_sect_params(self,nblock):
    """Build parameters for parallel job"""
    sect_pars={}

    for i in range(nblock):
      sect_pars[str(i)]=SEP.args.basic(name=str(i))
    return sect_pars
